--[[
	Description: Constants and tools used by DataMiner
	Copyright (C) 2014, 2015 Linuz@Proudmore.Eu
    This program is free software; you can redistribute it and/or modify it under the terms of the
	GNU General Public License as published by the Free Software Foundation; either version 2 of the License,
	or (at your option) any later version.
    This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along with this program;
	if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
--]]

-- Library references
local L = LibStub("AceLocale-3.0"):GetLocale("DataMiner", true);

-------------------------
-- Math, Date and Time --
-------------------------

--Java increment
function inc(i)
	return i + 1;
end

--Java decrement
function dec(i)
	return i - 1;
end

--not used
--function sleep(sec)
    --socket.select(nil, nil, sec)
--end

-- 1-based indexing
--function bit(p)
	--return 2 ^ (p - 1)
--end

-- Typical call: if hasbit(x, bit(3)) then
--function hasbit(x, p)
	--return x % (p + p) >= p;
--end

--function setbit(x, p)
	--return hasbit(x, p) and x or x + p;
--end

--function clearbit(x, p)
	--return hasbit(x, p) and x - p or x;
--end

-- Bitwise and (a & b)
-- We use this because bit.band seems to be deleted in patch 6.x
function xband( a, b)
    local p, c = 1, 0
    while a > 0 and b > 0 do
        local ra, rb = a%2, b%2
        if ra + rb > 1 then c = c + p end
        a, b, p = (a-ra)/2, (b-rb)/2, p*2
    end
    return c
end

-- Return the TimeSegment we are in
-- count = Segments per a day, so it should be a positive number
function TimeSegment (count)
	return floor( date("%H") / floor (24 / count) ) +1;
end

-- toggles a x-state variable
-- states are false-1-2-3-x
function ToggleX ( st, var )
	if (not var) then
		var = 1;
	elseif (var == dec(st)) then
		var = false;
	else
		var = inc(var)
	end
	return var;
end

-------------
-- Strings --
-------------

-- not used, Append a "\n" to the end
-- from Titan
--if ( string.sub(text, -1, -1) ~= "\n" ) then
	--text = text.."\n";
--end

-- Returns a string with first letter capitalized
function FirstUpper(str)
    return (str:gsub("^%l", string.upper))
end
		
-------------
-- Tables --
-------------

--Deletes the whole table
function Table_reset(array)
	for i, line in ipairs(array) do
		table.remove(line);
	end
end

-------------
-- Colours --
-------------

COL_BLACK  = "|cff000000"
COL_BLUE   = "|cff0000ff"
COL_GREEN  = "|cff66ff66"
COL_GREEN2 = "|c0070DE5f"
COL_RED    = "|cffff0000"
COL_GOLD   = "|cffffd700"
COL_ORANGE = "|cffff6600"
COL_YELLOW = "|cffffff00"
COL_WHITE  = "|cffffffff"

function GetHexColoredText(text, hexcolor)
	if (text and hexcolor) then
		return hexcolor..text.._G["FONT_COLOR_CODE_CLOSE"];
	end
end

-----------------
-- Professions --
-----------------
--PROF_BLACKSMITHING  = 164
--PROF_LEATHERWORKING = 165
--PROF_ALCHEMY        = 171
PROF_HERBALISM      = 182
PROF_MINING         = 186
--PROF_TAILORING      = 197
--PROF_ENGEINEERING   = 202
--PROF_ENCHANTING     = 333
--PROF_SKINNING       = 393
--PROF_JEWELCRAFTING  = 755
--PROF_INSCRIPTION    = 773

function HasProfession(prof, p)    --Parameters(PROF_* const, bool)
	local idx1, idx2, arch, fishing, cooking, firstaid = GetProfessions();
	local prof1, icon1, skillLv1, maxSkillLv1, numAbi1, spelloffset1, skillLine1;
	local prof2, icon2, skillLv2, maxSkillLv2, numAbi2, spelloffset2, skillLine2;
	if (idx1) then
		prof1, icon1, skillLv1, maxSkillLv1, numAbi1, spelloffset1, skillLine1 = GetProfessionInfo(idx1);
		if (prof1 and p) then
			print(prof1);          --Print to Chat for debug
		end
	end
	if (idx2) then
		prof2, icon2, skillLv2, maxSkillLv2, numAbi2, spelloffset2, skillLine2 = GetProfessionInfo(idx2);
		if (prof2 and p) then
			print(prof2);          --Print to Chat for debug
		end
	end
	if ((skillLine1 == prof) or (skillLine2 == prof)) then
		return true;
	end
	return false;
end

--Purpose: Test if Item is a Herb
--bagField - Bitwise OR of bag type
--0x0001 - Quiver (Köcher)
--0x0002 - Ammo Pouch (Patronentasche)
--0x0004 - Soul Bag
--0x0008 - Leatherworking Bag (Lederverarbeitung)
--0x0010 - Inscription Bag (Inschriftler)
--0x0020 - Herb Bag (Kräuter)
--0x0040 - Enchanting Bag (Verzauberer)
--0x0080 - Engineering Bag (Ing)
--0x0100 - Keyring (deleted)
--0x0200 - Gem Bag (Juwe)
--0x0400 - Mining Bag (Bergbau)
--0x0800 - Unused
--0x1000 - Vanity Pets
--0x2000 - Unused
--0x4000 - Unused
--0x8000 - Tackle Box
--0x10000 - Cooking Bag (Kochen)

function IsHerb(itemId)
	local bagField = GetItemFamily(itemId);
	if (xband(bagField, 0x0020) == 0x0020) then
		if (itemId == 22575) then
			--Living Particle, we are not interested in that item
			return nil;
		elseif (itemId == 22788) then
			--Flamecap, we are not interested in that item
			return nil;
		elseif (itemId == 22795) then
			--Devilsflower, we are not interested in that item
			return nil;
		elseif (itemId == 24245) then
			--Gluecap, we are not interested in that item
			return nil;
		elseif (itemId == 24401) then
			--Unknown Plantparts, we are not interested in that item
			return nil;
		elseif (itemId == 36908) then
			--Frozen Lotus, we are not interested in that item
			return nil;
		elseif (itemId == 37704) then
			-- Crystallized Living, we are not interested in that item
			return nil;
		elseif (itemId == 52329) then
			--Living, we are not interested in that item
			return nil;
		elseif (itemId == 63122) then
			--Livinggiving Saat, we are not interested in that item
			return nil;
		end
		return true;      -- The item will go to a herbalism-bag so its a herb
	end
	return nil;
end

function IsOre(itemId)
	local bagField = GetItemFamily(itemId);
	if (xband(bagField, 0x0400) == 0x0400) then
		if (itemId == 37701) then
			-- Crystallized Earth, we are not interested in that item
			return nil;
		elseif (itemId == 37702) then
			-- Crystallized Fire, we are not interested in that item
			return nil;
		elseif (itemId == 37705) then
			-- Crystallized Water, we are not interested in that item
			return nil;
		end
		return true;      --The item will go to a mining-bag so its ore
	end
	return nil;
end
